#include <avr/io.h>
#include "sdc.h"
#include "ssc.h"

//________________________________________________________________________________________________________________________________________
// Module name:			mmc.c 
// Compiler used:		avr-gcc 3.4.5
// Last Modifikation:	24.07.2007
// Version:				1.05
// Authors:				Stephan Busker 	
// Description:			Source files for connecting to an sdcard using the SSC
//
//........................................................................................................................................
// Functions:			unsigned char 			SDC_init(void);
//						unsigned char SDC_PutCommand (unsigned char *CMD);
//						unsigned char SDC_PutSector(unsigned long addr,unsigned char *Buffer);
//						unsigned char SDC_GetSector(unsigned long addr,unsigned char *Buffer);
//						void SDC_GetBlock(unsigned char *CMD,unsigned char *Buffer,unsigned int Bytes);
//
////........................................................................................................................................
// ext. functions:		extern void SSC_Init(void);
//						extern unsigned char SSC_GetChar (void);
//						extern void SSC_PutChar (unsigned char);
//						extern void SSC_Enable(void);
//						extern void SSC_Disable(void);
//........................................................................................................................................
//
// URL:					www.Mikro-Control.de
// mailto:				stephan.busker@mikro-control.de
//________________________________________________________________________________________________________________________________________




//________________________________________________________________________________________________________________________________________
// Funtion: 	SDC_Init(void);
// 
// Description:	This function initialises the SDCard to spi-mode. 
//				
//
// Returnvalue: the function returns 0 if the initialisation was successfull otherwise the function returns an errorcode.
//________________________________________________________________________________________________________________________________________

unsigned char SDC_Init(void)
{
	unsigned char Timeout = 0;
	unsigned char CMD[] = {0x40,0x00,0x00,0x00,0x00,0x95};
	
	SSC_Init();														// Initialise SSC to transmit data to the sdcard.

	for(unsigned int a=0;a<1050;a++) nop();						// short delay needed after poweronreset.

	for (unsigned char b = 0;b<0x0f;b++) 						// sending 74Clocks brings the sdcard into spimode.
	{
		SSC_PutChar(0xff);
	}
	
	
	while(SDC_PutCommand (CMD) !=1)							// Sending CMD0 (Reset) to the sdcard.
	{
		if (Timeout++ > 200)
			{
			
			return(1); 										
			}
	}
	
	Timeout = 0;
	CMD[0] = 0x41;													
	CMD[5] = 0xFF;
	while( SDC_PutCommand (CMD) !=0)							// Sending CMD1 to the sdcard.
	{
		if (Timeout++ > 100)
			{
			return(2); 
			}
	}

	SSC_Disable();													// disable sdcard.
	return(0);
}




//________________________________________________________________________________________________________________________________________
// Funtion: 	SDC_PutCommand(* CMD);
// 
// Description:	This function initialises the SDCard to spi-mode. 
//				
//
// Returnvalue: the function returns 0 if the initialisation was successfull otherwise the function returns an errorcode.
//________________________________________________________________________________________________________________________________________

unsigned char SDC_PutCommand (unsigned char *CMD)
{
	unsigned char tmp = 0xff;
	unsigned char Timeout = 0;

	SSC_Disable();														// disable chipselect
	SSC_PutChar(0xFF);													// Send 8 Clocks to the sdcard while card is not selected.
	SSC_Enable();														// enable chipselect.

    if (*CMD == 0x41) for(unsigned int a=0;a<2050;a++) nop();		// if command is CMD0 generate a short delay. 
	
	for (unsigned char a = 0;a<0x06;a++) 							// send the command sequence to the sdcard (6 bytes)
	{
		SSC_PutChar(*CMD++);
	}

	while (tmp == 0xff)												// Wait for response from sdcard.
	{;
		tmp = SSC_GetChar();
		if (Timeout++ > 100)
		{
			break; 													// or timeout.
		}
	}

	return(tmp);
}




//________________________________________________________________________________________________________________________________________
// Funtion: 	SDC_PutSector(void);
// 
// Description:	This function writes one sector of data to the SSC 
//				
//
// Returnvalue: none
//________________________________________________________________________________________________________________________________________

unsigned char SDC_PutSector(unsigned long addr,unsigned char *Buffer)
{
	unsigned char tmp;
	unsigned char CMD[] = {0x58,0x00,0x00,0x00,0x00,0xFF}; 
	
	addr = addr << 9; 													// convert sectoradress to byteadress
	
	CMD[1] = ((addr & 0xFF000000) >>24 );
	CMD[2] = ((addr & 0x00FF0000) >>16 );
	CMD[3] = ((addr & 0x0000FF00) >>8 );

	tmp = SDC_PutCommand (CMD);										// send command to sdcard.
	if (tmp != 0)
	{
		return(tmp);
	}
			
	for (unsigned char a=0;a<100;a++)								// wait until sdcard is ready
	{
		SSC_GetChar();
	}
	
	SSC_PutChar(0xFE);													// send start of header to the SSC	
	
	for (unsigned int a=0;a<512;a++)								// transmitt one sector (normaly 512bytes) of data to the sdcard.
	{
		SSC_PutChar(*Buffer++);
	}
	
	SSC_PutChar(0xFF); 													// write two bytes of crc to the sdcard (not used in spi-mode)
	SSC_PutChar(0xFF); 
	
	while (SSC_GetChar() != 0xff){};									// wait untile the sdcard is ready.
	
	SSC_Disable();														// disable sdcard.

	return(0);
}



//________________________________________________________________________________________________________________________________________
// Funtion: 	SDC_GetSector(unsigned long addr,unsigned char *Buffer);
// 
// Description:	This function reads one sector of data from the SSC
//				
//
// Returnvalue: none
//________________________________________________________________________________________________________________________________________

unsigned char SDC_GetSector(unsigned long addr,unsigned char *Buffer)
{	
	unsigned char CMD[] = {0x51,0x00,0x00,0x00,0x00,0xFF}; 
	
	addr = addr << 9; 													// convert sectoradress to byteadress.

	CMD[1] = ((addr & 0xFF000000) >>24 );
	CMD[2] = ((addr & 0x00FF0000) >>16 );
	CMD[3] = ((addr & 0x0000FF00) >>8 );

	SDC_GetBlock(CMD,Buffer,512);									// read specified sector from sdcard.

	return(0);
}



//________________________________________________________________________________________________________________________________________
// Funtion: 	SDC_GetBlock(void);
// 
// Description:	This function reads one block of data of int bytes from the SSC.
//				
//
// Returnvalue: the function returns 0 if the initialisation was successfull otherwise the function returns an errorcode.
//________________________________________________________________________________________________________________________________________

void SDC_GetBlock(unsigned char *CMD,unsigned char *Buffer,unsigned int Bytes)
{	
	if (SDC_PutCommand (CMD) != 0)										// Send command to the sdcard.
	{
		return;
	}
			
	while (SSC_GetChar() != 0xfe){};										// wait until the sdcard is ready to transmitt data.
	
	for (unsigned int a=0;a<Bytes;a++)									// read the block from the SSC (normaly 512Bytes)
	{
		*Buffer++ = SSC_GetChar();
	}
	
	SSC_GetChar();															// Read two bytes CRC- checksum (not used in spi-mode)
	SSC_GetChar();	
	SSC_Disable();															// disable sdcard.
}



